# coding: utf-8
from collections import OrderedDict

from django.core.exceptions import FieldDoesNotExist
from django.db.models.fields.related import ForeignKey
from django.db.models.fields.reverse_related import ForeignObjectRel


def get_field(model, name):
    u"""Возвращает поле с именем :arg:`name` модели :arg:`model`.

    Для обратных зависимостей в :arg:`name` нужно указывать имя атрибута
    объекта модели (см. :attr:`~django.db.models.ForeignKey.related_name`).

    :rtype: django.db.models.fields.Field or
        django.db.models.fields.related.RelatedField
    """
    for field in getattr(model, '_meta').get_fields():
        if isinstance(field, ForeignObjectRel):
            field_name = field.get_accessor_name()
        else:
            field_name = field.name

        if name == field_name:
            return field

    raise FieldDoesNotExist(name)


def get_nested_field(model, field_name):
    u"""Возвращает поле модели с учетом вложенности связей.

    Например, такой вызов вернет поле ``surname`` модели ``Person``, на которую
    есть внешний ключ из модели ``Employee`` через поле ``person``.

    .. code-block:: python

       >>> get_nested_field(Employee, 'person.surname')

    :rtype: django.db.models.fields.Field or
        django.db.models.fields.related.RelatedField
    """
    attr_name, _, nested_attr = field_name.partition('.')

    try:
        attr_value = model._meta.get_field(attr_name)
    except FieldDoesNotExist:
        # если это не поле, то возможно это обратная связь
        for field in model._meta.get_fields():
            if field.one_to_many and attr_name == field.get_accessor_name():
                if nested_attr:
                    attr_value = field.related_model
                else:
                    attr_value = field
                break
        else:
            raise

    if nested_attr:
        if isinstance(attr_value, ForeignKey):
            attr_value = attr_value.rel.to

        try:
            return get_nested_field(attr_value, nested_attr)
        except FieldDoesNotExist:
            raise FieldDoesNotExist(
                "{} has not field named '{}'"
                .format(model.__name__, field_name)
            )
    else:
        return attr_value


def get_columns_hierarchy(*columns):
    u"""Формирует иерахию полей по списку колонок.

    .. code-block:: python

       >>> get_columns_hierarchy('person.surname', 'person.firstname')
       {
           'person': {
               'surname': {},
               'firstname': {},
           }
       }

    :param columns: список строк с именами полей

    :rtype: collections.OrderedDict
    """

    result = OrderedDict()
    for column in columns:
        nodes = result
        for level_name in column.split('.'):
            if level_name not in nodes:
                nodes[level_name] = OrderedDict()
            nodes = nodes[level_name]

    return result


def get_field_value_by_display(field, display_value):
    u"""Возвращает значение поля с вариантами, соответствующее представлению.

    :param field: Поле модели.
    :type field: django.db.models.fields.Field

    :param display_value: Текстовое представление, соответствующее искомому
        значению поля.
    :type display_value: unicode

    :raises ValueError: если поле ``field`` не содержит вариантов допустимых
        значений, либо значения ``display_value`` нет среди допустимых
        вариантов.
    """
    if not field.choices:
        raise ValueError(u'Поле не содержит варианты выбора.')

    for value, display in field.choices:
        if display.lower() == display_value.lower():
            return value

    raise ValueError(
        u'Значения "{}" нет среди вариантов выбора.'
        .format(display_value)
    )
